// -*-c++-*-
//===========================================================
//=     University of Illinois at Urbana/Champaign          =
//=     Department of Computer Science                      =
//=     Dr. Dan Roth - Cognitive Computation Group          =
//=                                                         =
//=  Project: Snow                                          =
//=                                                         =
//=   Module: Target.h                                      =
//=  Version: 2.1                                           =
//=  Authors: Jeff Rosen and Andrew Carlson                 =
//=     Date: xx/xx/99                                      = 
//=                                                         =
//===========================================================

#ifndef __TARGET_H__
#define __TARGET_H__

#include "Feature.h"
#include "LearningAlgorithm.h"
#include <set>
#include <map>
#include <vector>

using namespace std;

typedef vector<LearningAlgorithm*> AlgorithmVector;

extern int examples;

class Target
{
public:
    Target();
    Target( FeatureID id, LearningAlgorithm* pAlg );

    bool PresentExample( Example& ex );
    void TrainingComplete();
    void Discard();
    void Discard( FeatureID id, double delta );

    bool operator==( const Target& rhs ) const;
    bool operator!=( const Target& rhs ) const;
    bool operator<( const Target& rhs ) const;
    bool operator<=( const Target& rhs ) const;
    bool operator>( const Target& rhs ) const;
    bool operator>=( const Target& rhs ) const;

    void ShowStatistics( Counter total ) const;
    void ShowFeatures() const;

    LearningAlgorithm* PAlgorithm() const;
    FeatureID   Id() const;
    bool        ExternalActivation() const;
    double      InternalActivation() const;
    double		Confidence() const;
    void        NormalizeConfidence(double totalConfidence);
    double      PriorProbability() const ;
    void        CollectFeatures( set<FeatureID>& featureSet ) const;
    double      WeightOfFeature( FeatureID id ) const;
    void        ResetCounters();

    void Read( ifstream& in , AlgorithmVector& algorithms);
    void Write( ofstream& out );
    
private:
    typedef map<FeatureID, MinimalFeature> FeatureMap;
    
    FeatureID   targetID;
    bool        externalActivation;
    double      internalActivation;
    FeatureMap  features;
    Counter     activeFeatureCount;
    Counter     activeCount;
    Counter     nonActiveCount;
    double      priorProbability;
    double      confidence;
    double      discardThreshold;

    LearningAlgorithm*  pAlgorithm;

    friend class LearningAlgorithm;
    friend class Winnow;
    friend class Perceptron;
    friend class NaiveBayes;
};

inline Target::Target() :
        targetID((FeatureID)-1),
        externalActivation(false),
        internalActivation(0.0),
        activeFeatureCount(0L),
        activeCount(0L),
        nonActiveCount(0L),
        priorProbability(1.0),
        confidence(1.0),
        discardThreshold(0.0),
        pAlgorithm(NULL)
{
}

inline Target::Target( FeatureID id, LearningAlgorithm* pAlg ) :
        targetID(id),
        externalActivation(false),
        internalActivation(0.0),
        activeFeatureCount(1L),
        activeCount(0L),
        nonActiveCount(0L),
        priorProbability(1.0),
        confidence(1.0),
        discardThreshold(0.0),
        pAlgorithm(pAlg)
{
    features.insert(make_pair(FIXED_FEATURE_ID, MinimalFeature()));
    features[FIXED_FEATURE_ID].discarded = active;
    features[FIXED_FEATURE_ID].activeCount = 0;
    features[FIXED_FEATURE_ID].weight = pAlg->defaultWeight;
}

inline bool Target::PresentExample( Example& ex )
{
    return pAlgorithm->PresentExample(*this, ex);
}

inline void Target::TrainingComplete()
{
    priorProbability = activeCount / (double) examples;
    pAlgorithm->TrainingComplete(*this);
}

inline LearningAlgorithm* Target::PAlgorithm() const
{
	return pAlgorithm;
}

inline FeatureID Target::Id() const
{
    return targetID;
}

inline bool Target::ExternalActivation() const
{
	return externalActivation;
}

inline double Target::InternalActivation() const
{
	return internalActivation;
}


inline double Target::Confidence() const
{
	return confidence;
}

inline void Target::NormalizeConfidence(double totalConfidence) 
{   
    confidence /= totalConfidence;
}

inline double Target::PriorProbability() const
{
	return priorProbability;
}

inline void Target::CollectFeatures( set<FeatureID>& featureSet ) const
{
    FeatureMap::const_iterator it = features.begin();
    for ( ; it != features.end(); it++)
    {
        featureSet.insert(it->first);
    }
}

inline double Target::WeightOfFeature( FeatureID id ) const
{
    FeatureMap::const_iterator it = features.find(id);
    if (it != features.end())
    {
        if (it->second.discarded == active)
        {
            return it->second.weight;
        }
    }

    return 0.0;
}

inline void Target::ResetCounters()
{
    activeCount = 0L;
    nonActiveCount = 0L;
 
    externalActivation = false;
    internalActivation = 0.0;
}


#endif


